package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.BasePriceStrategyDetailEnt;
import pps.core.base.entity.BasePriceStrategyEnt;
import pps.core.base.entity.BasePriceStrategyMonthEnt;
import pps.core.base.entity.BasePriceStrategyView;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_price_strategy.*;
import pps.core.base.service.data.base_price_strategy_month.CreateBasePriceStrategyMonthInput;
import pps.core.base.service.data.base_price_strategy_month.UpdateBasePriceStrategyMonthInput;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 市电峰谷配置模块
 *
 * @author ZWT
 * @date 2023/08/24
 */
@XService
@XText("市电峰谷配置模块")
public class BasePriceStrategyService {

    /**
     * 创建市电峰谷策略
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("创建市电峰谷策略")
    public XServiceResult createBasePriceStrategy(XContext context, CreateBasePriceStrategyInput input) {
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            BasePriceStrategyEnt entity = new BasePriceStrategyEnt();
            XCopyUtils.copyObject(input, entity);
            this.setBasePriceStrategyEntDefault(entity, session);
            mapper.insert(entity);
            String strategyId = entity.getId();
            List<CreateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (!inputMonths.isEmpty()) {
                List<BasePriceStrategyMonthEnt> monthList = new ArrayList<>(16);
                XCopyUtils.copyList(inputMonths, monthList, BasePriceStrategyMonthEnt.class);
                this.saveStrategyMonthDetail(context, monthList, strategyId, session);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 更新市电峰谷策略
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("更新市电峰谷策略")
    public XServiceResult updateBasePriceStrategy(XContext context, UpdateBasePriceStrategyInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        String strategyId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
            QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, strategyId);
            BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            List<UpdateBasePriceStrategyMonthInput> inputMonths = input.getMonths();
            if (!inputMonths.isEmpty()) {
                List<BasePriceStrategyMonthEnt> monthList = new ArrayList<>(16);
                XCopyUtils.copyList(inputMonths, monthList, BasePriceStrategyMonthEnt.class);
                //删除月信息
                BasePriceStrategyMonthViewMapper monthViewMapper = context.getBean(BasePriceStrategyMonthViewMapper.class);
                monthViewMapper.deleteBatchByStrategyMonth(strategyId, monthList);
                //删除详情
                BasePriceStrategyDetailViewMapper detailViewMapper = context.getBean(BasePriceStrategyDetailViewMapper.class);
                detailViewMapper.deleteBatchByStrategyMonth(strategyId, monthList);
                this.saveStrategyMonthDetail(context, monthList, strategyId, session);
            }
            XCopyUtils.copyObject(input, entity);
            this.setBasePriceStrategyEntDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePriceStrategy(XContext context, DeleteBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, input.getId());
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePriceStrategyOutput> getBasePriceStrategy(XContext context, GetBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePriceStrategyEnt::getId, input.getId());
        BasePriceStrategyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyOutput output = new GetBasePriceStrategyOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePriceStrategyOutput> queryBasePriceStrategy(XContext context, QueryBasePriceStrategyInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        QueryWrapper<BasePriceStrategyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePriceStrategyEnt::getId, input.getId())
                .eq(BasePriceStrategyEnt::getIsDeleted, input.getIsDeleted())
                .like(BasePriceStrategyEnt::getCreateById, input.getCreateById())
                .like(BasePriceStrategyEnt::getCreateByName, input.getCreateByName())
                .eq(BasePriceStrategyEnt::getCreateTime, input.getCreateTime())
                .like(BasePriceStrategyEnt::getModifyById, input.getModifyById())
                .like(BasePriceStrategyEnt::getModifyByName, input.getModifyByName())
                .eq(BasePriceStrategyEnt::getModifyTime, input.getModifyTime())
                .like(BasePriceStrategyEnt::getPolicyName, input.getPolicyName())
                .like(BasePriceStrategyEnt::getAreaCode, input.getAreaCode())
                .eq(BasePriceStrategyEnt::getStrategyYear, input.getStrategyYear())
                .eq(BasePriceStrategyEnt::getIsFixedPrice, input.getIsFixedPrice())
                .eq(BasePriceStrategyEnt::getFixedElectricityPrice, input.getFixedElectricityPrice());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePriceStrategyOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePriceStrategyOutput> dynamicQueryBasePriceStrategy(XContext context, CustomQueryInput input) {
        BasePriceStrategyMapper mapper = context.getBean(BasePriceStrategyMapper.class);
        return XMapperHelper.query(mapper, input, BasePriceStrategyEnt.class, QueryBasePriceStrategyOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePriceStrategyViewOutput> getBasePriceStrategyView(XContext context, GetBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView record = new BasePriceStrategyView();
        XCopyUtils.copyObject(input, record);
        BasePriceStrategyView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePriceStrategyViewOutput output = new GetBasePriceStrategyViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePriceStrategyViewOutput> queryBasePriceStrategyView(XContext context, QueryBasePriceStrategyViewInput input) {
        BasePriceStrategyViewMapper mapper = context.getBean(BasePriceStrategyViewMapper.class);
        BasePriceStrategyView record = new BasePriceStrategyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePriceStrategyView> list = mapper.selectList(record);
        PageInfo<BasePriceStrategyView> pageInfo = new PageInfo<>(list);
        List<QueryBasePriceStrategyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePriceStrategyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 保存策略明细
     *
     * @param context    上下文
     * @param monthList  月清单
     * @param strategyId 战略id
     * @param session    会话
     */
    private void saveStrategyMonthDetail(XContext context, List<BasePriceStrategyMonthEnt> monthList, String strategyId, PpsUserSession session) {
        BasePriceStrategyMonthMapper monthMapper = context.getBean(BasePriceStrategyMonthMapper.class);
        BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
        Integer strategyMonth;
        List<BasePriceStrategyDetailEnt> detailList;
        for (BasePriceStrategyMonthEnt monthEnt : monthList) {
            this.setBasePriceStrategyMonthEntDefault(monthEnt, strategyId, session);
            monthMapper.insert(monthEnt);
            strategyMonth = monthEnt.getStrategyMonth();
            detailList = monthEnt.getDetails();
            for (BasePriceStrategyDetailEnt detailEnt : detailList) {
                this.setBasePriceStrategyDetailEntDefault(detailEnt, strategyId, strategyMonth, session);
                detailMapper.insert(detailEnt);
            }
        }
    }

    /**
     * 设置市电峰谷当月明细配置默认值
     *
     * @param entity        实体
     * @param strategyId    战略id
     * @param strategyMonth 战略月
     * @param session       会话
     */
    private void setBasePriceStrategyDetailEntDefault(BasePriceStrategyDetailEnt entity, String strategyId, Integer strategyMonth, PpsUserSession session) {
        Date now = new Date();
        entity.setStrategyId(strategyId);
        entity.setStrategyMonth(strategyMonth);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(now);
    }

    /**
     * 设置市电峰谷月配置默认值
     *
     * @param entity     实体
     * @param strategyId 战略id
     * @param session    会话
     */
    private void setBasePriceStrategyMonthEntDefault(BasePriceStrategyMonthEnt entity, String strategyId, PpsUserSession session) {
        Date now = new Date();
        entity.setStrategyId(strategyId);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(now);
    }

    /**
     * 设置市电峰谷策略默认值
     *
     * @param entity  实体
     * @param session 会话
     */
    private void setBasePriceStrategyEntDefault(BasePriceStrategyEnt entity, PpsUserSession session) {
        String id = entity.getId();
        Date now = new Date();
        String userId = session.getId();
        String userName = session.getUserName();
        if (StringUtils.isEmpty(id)) {
            entity.setCreateById(userId);
            entity.setCreateByName(userName);
            entity.setCreateTime(now);
        }
        entity.setModifyById(userId);
        entity.setModifyByName(userName);
        entity.setModifyTime(now);
    }
}
